extends "res://Scripts/StoryLib.gd"

onready var story_window : = $StoryWindow
onready var bg_image : = $StoryWindow/BGImage
onready var bg_mirror : = $StoryWindow/BGMirror
onready var left_image : = $StoryWindow/LeftImage
onready var left_mirror : = $StoryWindow/LeftMirror
onready var right_image : = $StoryWindow/RightImage
onready var right_mirror : = $StoryWindow/RightMirror
onready var center_image : = $StoryWindow/CenterImage
onready var center_mirror : = $StoryWindow/CenterMirror
onready var front_image : = $StoryWindow/FrontImage
onready var front_mirror : = $StoryWindow/FrontMirror
onready var left_emote : = $StoryWindow/LeftEmote
onready var left_emote_mirror : = $StoryWindow/LeftEmoteMirror
onready var right_emote : = $StoryWindow/RightEmote
onready var right_emote_mirror : = $StoryWindow/RightEmoteMirror
onready var center_emote : = $StoryWindow/CenterEmote
onready var center_emote_mirror : = $StoryWindow/CenterEmoteMirror
onready var fade_mask : = $StoryWindow/FadeMask
onready var story_label : = $StoryWindow/StoryLabel
onready var page_arrow : = $StoryWindow/PageArrow

onready var input_window : = $InputWindow
onready var next_button : = $InputWindow/NextButton
onready var next_button_image : = $InputWindow/NextButton/Image
onready var choice_buttons : = $InputWindow/ChoiceButtons
onready var choice_buttons_a : = $InputWindow/ChoiceButtons/A
onready var choice_buttons_b : = $InputWindow/ChoiceButtons/B
onready var choice_button_a1 : = $InputWindow/ChoiceButtons/A/Button1
onready var choice_button_a2 : = $InputWindow/ChoiceButtons/A/Button2
onready var choice_button_a3 : = $InputWindow/ChoiceButtons/A/Button3
onready var choice_label_a1 : = $InputWindow/ChoiceButtons/A/Button1/Label
onready var choice_label_a2 : = $InputWindow/ChoiceButtons/A/Button2/Label
onready var choice_label_a3 : = $InputWindow/ChoiceButtons/A/Button3/Label
onready var choice_button_b1 : = $InputWindow/ChoiceButtons/B/Button1
onready var choice_button_b2 : = $InputWindow/ChoiceButtons/B/Button2
onready var choice_button_b3 : = $InputWindow/ChoiceButtons/B/Button3
onready var choice_button_b4 : = $InputWindow/ChoiceButtons/B/Button4
onready var choice_button_b5 : = $InputWindow/ChoiceButtons/B/Button5
onready var choice_button_b6 : = $InputWindow/ChoiceButtons/B/Button6
onready var choice_label_b1 : = $InputWindow/ChoiceButtons/B/Button1/Label
onready var choice_label_b2 : = $InputWindow/ChoiceButtons/B/Button2/Label
onready var choice_label_b3 : = $InputWindow/ChoiceButtons/B/Button3/Label
onready var choice_label_b4 : = $InputWindow/ChoiceButtons/B/Button4/Label
onready var choice_label_b5 : = $InputWindow/ChoiceButtons/B/Button5/Label
onready var choice_label_b6 : = $InputWindow/ChoiceButtons/B/Button6/Label

onready var skip_button : = $SkipButton
onready var end_credits : = $EndCredits
onready var all_mask : = $AllMask
onready var debug_menu : = $DebugMenu
onready var debug_page_debugger : = $DebugMenu/PageDebugger
onready var debug_page_label : = $DebugMenu/PageDebugger/LineEdit
onready var debug_skip_button : = $DebugMenu/SkipButton

onready var anim : = $AnimationPlayer
onready var anim_left : = $AnimLeft
onready var anim_right : = $AnimRight
onready var anim_center : = $AnimCenter
onready var anim_end_credits : = $AnimEndCredits

func _ready():
	clear_images()
	all_mask.show()
	page_arrow.hide()
	end_credits.hide()
	fade_mask.hide()
	choice_buttons.hide()
	skip_button.hide()
	next_button_image.hide()
	input_window.show()
	next_button.show()
	current_story = Data.current_story
	page = "0"
	skip_marker = ""
	choice_marker1 = ""
	choice_marker2 = ""
	choice_marker3 = ""
	choice_marker4 = ""
	choice_marker5 = ""
	choice_marker6 = ""
	enabled_text_smooth_animation = true
	story_label.set_text("")
	
	### debug mode
	if is_story_debug_mode:
		debug_menu.show()
		debug_page_label.set_text(page)
	else:
		debug_menu.hide()
	
	# セーブデータ読み込み
	load_save()
	
	anim.play("fadein_window")
	yield(get_tree().create_timer(1.0), t)
	get_next()

func get_next():
	skip_button.disabled = true
	all_mask.show()
	next_button_image.hide()
	page_arrow.hide()
	
	if current_story == "" || current_story == null:
		# フリーズ回避のためオープニングを挿入する
		current_story = "opening"
	
	var words = get_line(current_story, page)
	
	# if文エラー回避のために、テキスト内の整数や少数をstringに変換する
	if typeof(words) == 3: # float/real
		words = str(words)
	
	# ジャンプマーカーを識別する
	# マーカーが見つからない場合、エラーないしスタックするので注意
	if jump_marker_is_valid && (jump_marker != "" || jump_marker != null):
		if words.left(1) == "@":
			var trimmed = words.lstrip("@")
			var word = trimmed.split(" ", true, 1)
			
			# マーカー名が一致した場合、次の行から開始する
			if word[0] == jump_marker:
				jump_marker = ""
				jump_marker_is_valid = false
		
		return next()
	
	# コマンドを識別する
	if words.left(1) == "[":
		var trimmed = words.lstrip("[")
		var word = trimmed.split(" ", true, 1)
		
		match word[0]:
			# x秒待機する
			# [wait seconds
			"wait":
				yield(get_tree().create_timer(1.0 * float(word[1])), t)
			# 画像を表示する
			"bg":
				set_image(bg_image, bg_mirror, word[1])
			"left":
				set_image(left_image, left_mirror, word[1])
			"right":
				set_image(right_image, right_mirror, word[1])
			"center":
				set_image(center_image, center_mirror, word[1])
			"front":
				set_image(front_image, front_mirror, word[1])
			"left_emote":
				set_image(left_emote, left_emote_mirror, word[1])
			"right_emote":
				set_image(right_emote, right_emote_mirror, word[1])
			"center_emote":
				set_image(center_emote, center_emote_mirror, word[1])
			# 画像を横軸に反転する
			"h_bg":
				flip_image(bg_image, bg_mirror, true)
			"h_left":
				flip_image(left_image, left_mirror, true)
			"h_right":
				flip_image(right_image, right_mirror, true)
			"h_center":
				flip_image(center_image, center_mirror, true)
			"h_front":
				flip_image(front_image, front_mirror, true)
			"h_left_emote":
				flip_image(left_emote, left_emote_mirror, true)
			"h_right_emote":
				flip_image(right_emote, right_emote_mirror, true)
			"h_center_emote":
				flip_image(center_emote, center_emote_mirror, true)
			# 画像を縦軸に反転する
			"v_bg":
				flip_image(bg_image, bg_mirror, false)
			"v_left":
				flip_image(left_image, left_mirror, false)
			"v_right":
				flip_image(right_image, right_mirror, false)
			"v_center":
				flip_image(center_image, center_mirror, false)
			"v_front":
				flip_image(front_image, front_mirror, false)
			"v_left_emote":
				flip_image(left_emote, left_emote_mirror, false)
			"v_right_emote":
				flip_image(right_emote, right_emote_mirror, false)
			"v_center_emote":
				flip_image(center_emote, center_emote_mirror, false)
			# 画像を削除する
			"c_left":
				texture_fade_clear(left_image)
			"c_right":
				texture_fade_clear(right_image)
			"c_center":
				texture_fade_clear(center_image)
			"c_front":
				texture_fade_clear(front_image)
			"c_left_emote":
				texture_fade_clear(left_emote)
			"c_right_emote":
				texture_fade_clear(right_emote)
			"c_center_emote":
				texture_fade_clear(center_emote)
			"c_emotes":
				texture_fade_clear(left_emote)
				texture_fade_clear(right_emote)
				texture_fade_clear(center_emote)
			"c_pd":
				texture_fade_clear(left_image)
				texture_fade_clear(right_image)
				texture_fade_clear(center_image)
				texture_fade_clear(left_emote)
				texture_fade_clear(right_emote)
				texture_fade_clear(center_emote)
			"c_all":
				texture_fade_clear(left_image)
				texture_fade_clear(right_image)
				texture_fade_clear(center_image)
				texture_fade_clear(front_image)
				texture_fade_clear(left_emote)
				texture_fade_clear(right_emote)
				texture_fade_clear(center_emote)
			# 画像を1秒かけてフェードインし、全体で2秒待機する
			"fadein_bg":
				image_fadein(bg_image, bg_mirror, word[1], anim, "fadein_background")
				yield(get_tree().create_timer(fadein_timer), t)
				set_image(bg_image, bg_mirror, word[1])
			"fadein_left":
				image_fadein(left_image, left_mirror, word[1], anim_left, "fadein_left")
				yield(get_tree().create_timer(fadein_timer), t)
				set_image(left_image, left_mirror, word[1])
			"fadein_right":
				image_fadein(right_image, right_mirror, word[1], anim_right, "fadein_right")
				yield(get_tree().create_timer(fadein_timer), t)
				set_image(right_image, right_mirror, word[1])
			"fadein_center":
				image_fadein(center_image, center_mirror, word[1], anim_center, "fadein_center")
				yield(get_tree().create_timer(fadein_timer), t)
				set_image(center_image, center_mirror, word[1])
			"fadein_front":
				image_fadein(front_image, front_mirror, word[1], anim, "fadein_front")
				yield(get_tree().create_timer(fadein_timer), t)
				set_image(front_image, front_mirror, word[1])
			"fadein_left_emote":
				image_fadein(left_emote, left_emote_mirror, word[1], anim_left, "fadein_left_emote")
				yield(get_tree().create_timer(fadein_timer), t)
				set_image(left_emote, left_emote_mirror, word[1])
			"fadein_right_emote":
				image_fadein(right_emote, right_emote_mirror, word[1], anim_right, "fadein_right_emote")
				yield(get_tree().create_timer(fadein_timer), t)
				set_image(right_emote, right_emote_mirror, word[1])
			"fadein_center_emote":
				image_fadein(center_emote, center_emote_mirror, word[1], anim_center, "fadein_center_emote")
				yield(get_tree().create_timer(fadein_timer), t)
				set_image(center_emote, center_emote_mirror, word[1])
			"fadein_all":
				anim.play("fadein_all")
				yield(get_tree().create_timer(fadein_timer), t)
				fade_mask.hide()
			"fadein_window":
				anim.play("fadein_window")
				yield(get_tree().create_timer(1.0), t)
			# 上記のフェードアウト版
			"fadeout_bg":
				anim.play("fadeout_background")
				yield(get_tree().create_timer(fadein_timer), t)
				texture_fade_clear(bg_image)
			"fadeout_left":
				anim_left.play("fadeout_left")
				yield(get_tree().create_timer(fadein_timer), t)
				texture_fade_clear(left_image)
			"fadeout_right":
				anim_right.play("fadeout_right")
				yield(get_tree().create_timer(fadein_timer), t)
				texture_fade_clear(right_image)
			"fadeout_center":
				anim_center.play("fadeout_center")
				yield(get_tree().create_timer(fadein_timer), t)
				texture_fade_clear(center_image)
			"fadeout_front":
				anim.play("fadeout_front")
				yield(get_tree().create_timer(fadein_timer), t)
				texture_fade_clear(front_image)
			"fadeout_left_emote":
				anim_left.play("fadeout_left_emote")
				yield(get_tree().create_timer(fadein_timer), t)
				texture_fade_clear(left_emote)
			"fadeout_right_emote":
				anim_right.play("fadeout_right_emote")
				yield(get_tree().create_timer(fadein_timer), t)
				texture_fade_clear(right_emote)
			"fadeout_center_emote":
				anim_center.play("fadeout_center_emote")
				yield(get_tree().create_timer(fadein_timer), t)
				texture_fade_clear(center_emote)
			"fadeout_emotes":
				anim_left.play("fadeout_left_emote")
				anim_right.play("fadeout_right_emote")
				anim_center.play("fadeout_center_emote")
				yield(get_tree().create_timer(fadein_timer), t)
				texture_fade_clear(left_emote)
				texture_fade_clear(right_emote)
				texture_fade_clear(center_emote)
			"fadeout_pd":
				anim_left.play("fadeout_left")
				anim_right.play("fadeout_right")
				anim_center.play("fadeout_center")
				yield(get_tree().create_timer(fadein_timer), t)
				texture_fade_clear(left_image)
				texture_fade_clear(right_image)
				texture_fade_clear(center_image)
			"fadeout_all":
				fade_mask.show()
				anim.play("fadeout_all")
				yield(get_tree().create_timer(fadein_timer), t)
			"fadeout_window":
				anim.play("fadeout_window")
				yield(get_tree().create_timer(1.0), t)
			# BGMを再生する
			"play_bgm":
				Data.emit_signal("audio_bgm", word[1], true, false)
			# BGMを止める
			"stop_bgm":
				Data.emit_signal("audio_bgm", null, false, false)
			# BGMを3.5秒かけてフェードアウトさせる
			"fadeout_bgm":
				Data.emit_signal("audio_bgm", null, false, true)
				yield(get_tree().create_timer(3.5), t)
			# SEを再生する
			"play_se":
				Data.emit_signal("audio_se", word[1], true, false)
			# 0.5秒のフラッシュ
			"flash":
				anim.play("flash")
				yield(get_tree().create_timer(0.5), t)
			# テキストラベルを白紙にする
			"c":
				story_label.set_text("")
			# 選択肢を2つ出現させる
			# [choice2 label1, marker1, label2, marker2
			"choice2":
				var field = word[1].split(", ", true, 3)
				choice_label_a1.set_text(field[0])
				choice_marker1 = field[1]
				choice_label_a2.set_text(field[2])
				choice_marker2 = field[3]
				return show_choice_menu(2)
			# 選択肢を3つ出現させる
			# [choice3 label1, marker1, label2, marker2, label3, marker3
			"choice3":
				var field = word[1].split(", ", true, 5)
				choice_label_a1.set_text(field[0])
				choice_marker1 = field[1]
				choice_label_a2.set_text(field[2])
				choice_marker2 = field[3]
				choice_label_a3.set_text(field[4])
				choice_marker3 = field[5]
				return show_choice_menu(3)
			# 選択肢を4つ出現させる
			# [choice4 label1, marker1, label2, marker2, label3, marker3, label4, marker4
			"choice4":
				var field = word[1].split(", ", true, 7)
				choice_label_b1.set_text(field[0])
				choice_marker1 = field[1]
				choice_label_b2.set_text(field[2])
				choice_marker2 = field[3]
				choice_label_b3.set_text(field[4])
				choice_marker3 = field[5]
				choice_label_b4.set_text(field[6])
				choice_marker4 = field[7]
				return show_choice_menu(4)
			# 選択肢を5つ出現させる
			# [choice5 label1, marker1, label2, marker2, label3, marker3, label4, marker4, label5, marker5
			"choice5":
				var field = word[1].split(", ", true, 9)
				choice_label_b1.set_text(field[0])
				choice_marker1 = field[1]
				choice_label_b2.set_text(field[2])
				choice_marker2 = field[3]
				choice_label_b3.set_text(field[4])
				choice_marker3 = field[5]
				choice_label_b4.set_text(field[6])
				choice_marker4 = field[7]
				choice_label_b5.set_text(field[8])
				choice_marker5 = field[9]
				return show_choice_menu(5)
			# 選択肢を6つ出現させる
			# [choice6 label1, marker1, label2, marker2, label3, marker3, label4, marker4, label5, marker5, label6, marker7
			"choice6":
				var field = word[1].split(", ", true, 11)
				choice_label_b1.set_text(field[0])
				choice_marker1 = field[1]
				choice_label_b2.set_text(field[2])
				choice_marker2 = field[3]
				choice_label_b3.set_text(field[4])
				choice_marker3 = field[5]
				choice_label_b4.set_text(field[6])
				choice_marker4 = field[7]
				choice_label_b5.set_text(field[8])
				choice_marker5 = field[9]
				choice_label_b6.set_text(field[10])
				choice_marker6 = field[11]
				return show_choice_menu(6)
			# if分岐
			#[if_not var_name, var, marker
			"if":
				var field = word[1].split(", ", true, 2)
				var f1 = str(field[1])
				
				# bool判定の不作動回避
				if f1 == "True" || f1 == "true":
					f1 = "True"
				elif f1 == "False" || f1 == "false":
					f1 = "False"
				
				if str(temporary_saves[field[0]]) == f1:
					jump(field[2])
					
					if is_story_debug_mode:
						print("debug: ", "successed in ", str(field[0]), " == ", f1)
				else:
					if is_story_debug_mode:
						print("debug: ", "not successed in ", str(field[0]), " == ", f1)
			# if not分岐
			# [if_not var_name, var, marker
			"if_not":
				var field = word[1].split(", ", true, 2)
				var f1 = str(field[1])
				
				# bool判定の不作動回避
				if f1 == "True" || f1 == "true":
					f1 = "True"
				elif f1 == "False" || f1 == "false":
					f1 = "False"
				
				if str(temporary_saves[field[0]]) != f1:
					jump(field[2])
					
					if is_story_debug_mode:
						print("debug: ", "successed in ", str(field[0]), " != ", f1)
				else:
					if is_story_debug_mode:
						print("debug: ", "not successed in ", str(field[0]), " != ", f1)
			# セーブデータに関する命令
			# セーブデータの再読み込みもしくは新規作成
			"load_save":
				load_save()
			# セーブデータを消す
			"clear_save":
				clear_save()
			# int変数の更新
			# [set_int var_name, int
			"set_int":
				var field = word[1].split(", ", true, 1)
				temporary_saves[field[0]] = int(field[1])
				update_save(field[0], int(field[1]))
				
				if is_story_debug_mode:
					print("debug: ", field[0], " = ", field[1])
			# boolとstring変数の更新
			# [set var_name, var
			"set":
				var field = word[1].split(", ", true, 1)
				var f = field[1]
				
				# bool判定の不作動回避
				if f == "True" || f == "true" :
					f = true
				elif f == "False" || f == "false" :
					f = false
				
				temporary_saves[field[0]] = f
				update_save(field[0], f)
				
				if is_story_debug_mode:
					print("debug: ", field[0], " = ", f)
			# 指定マーカーへジャンプ
			"goto":
				jump(word[1])
			# 別テキストファイルの最初にジャンプ
			"next":
				Data.current_story = word[1]
				current_story = Data.current_story
				page = "0"
			# 次のシーンへ
			"next_scene":
				Data.emit_signal("node", word[1], true)
			# 特定の変数が指定通りの場合にスキップボタンを表示する
			"set_skip":
				var field = word[1].split(", ", true, 2)
				var f1 = str(field[1])
				
				# bool判定の不作動回避
				if f1 == "True" || f1 == "true":
					f1 = "True"
				elif f1 == "False" || f1 == "false":
					f1 = "False"
				
				if str(temporary_saves[field[0]]) == f1:
					skip_marker = str(field[2])
					skip_button.show()
			# ストーリーウインドウを閉じて配置をリセットする
			"end":
				return queue_free()
			# テキストのスムーズ表示を有効にする
			"enable_smooth_text":
				enabled_text_smooth_animation = true
			# テキストのスムーズ表示を無効にする
			"disable_smooth_text":
				enabled_text_smooth_animation = false
			# 【β版の機能】エンドクレジットを再生する
			# BB Code Textに記述してください
			# クレジットの長さに応じて、AnimEndCreditsノードの
			# "start"の再生時間とクレジット用ノードの位置を調整してください
			"start_end_credits":
				skip_button.hide()
				anim_end_credits.play("stand_by")
				end_credits.show()
				anim.play("fadein_window")
				yield(get_tree().create_timer(1.0), t)
				anim_end_credits.play("start")
				yield(get_tree().create_timer(anim_end_credits.get_current_animation_length()), t)
			# 【β版の機能】エンドクレジットを終了する
			"end_end_credits":
				anim.play("fadeout_window")
				yield(get_tree().create_timer(1.0), t)
				end_credits.hide()
		return next()
	# 順番に抜けがあったりコメント行の場合、自動的にその行を飛ばす
	elif words.left(2) == "//" || words == null || words == "":
		return next()
	# テキストを表示する
	else:
		# テキスト表示をスムーズに
		if enabled_text_smooth_animation:
			var character_count = 0
			# これまでのテキストを一時保存する
			var temp_words = story_label.get_text()
			#.txtで読み込んだ際に\nを正しく認識させる
			var trim_words = words.c_unescape()
			
			# character_countがtrim_words以上になるまで繰り返す
			while character_count <= trim_words.length():
				var character = trim_words.left(character_count)
				var all_words = temp_words + character
				character_count += 1
				# temp_wordsと合わせて、一文字ずつ0.02秒間隔で表示させる
				story_label.set_bbcode(all_words)
				yield(get_tree().create_timer(0.02), t)
		else:
			story_label.set_bbcode(story_label.get_text() + words.c_unescape())
		
		page_arrow.show()
		anim.play("page_arrow_on")
		
		if enabled_skip_button:
			skip_button.disabled = false
		
		all_mask.hide()
		next_button_image.show()
		return add_page()

func jump(field):
	jump_marker = field
	jump_marker_is_valid = true
	page = "0"

func next():
	add_page()
	get_next()

func show_choice_menu(choices):
	if choices <= 3:
		choice_buttons_b.hide()
		choice_buttons_a.show()
		choice_button_a1.show()
		choice_button_a2.show()
		choice_button_a3.show()
	else:
		choice_buttons_a.hide()
		choice_buttons_b.show()
		choice_button_b1.show()
		choice_button_b2.show()
		choice_button_b3.show()
		choice_button_b4.show()
		choice_button_b5.show()
		choice_button_b6.show()
	
	if choices == 2:
		choice_button_a3.hide()
	elif choices == 4:
		choice_button_b5.hide()
		choice_button_b6.hide()
	elif choices == 5:
		choice_button_b6.hide()
	
	next_button.hide()
	choice_buttons.show()
	all_mask.hide()

func choice_button(choice_number):
	match choice_number:
		1:
			jump_marker = choice_marker1
		2:
			jump_marker = choice_marker2
		3:
			jump_marker = choice_marker3
		4:
			jump_marker = choice_marker4
		5:
			jump_marker = choice_marker5
		6:
			jump_marker = choice_marker6
	
	choice_buttons.hide()
	next_button.show()
	jump(jump_marker)
	get_next()

func skip_button():
	if skip_marker != "" || skip_marker != null:
		jump(skip_marker)
		get_next()
		skip_marker = ""
		# 押されたスキップボタンを非表示にする
		skip_button.hide()
		# 選択肢を非表示にし、タッチで次へを表示する
		choice_buttons.hide()
		next_button.show()
		
		if is_story_debug_mode:
			debug_skip_button.hide()

func texture_fade_clear(texture):
	texture.set_texture(null)
	texture.modulate = Color(1, 1, 1, 1)
	# 向きリセット
	if texture.is_flipped_h():
		texture.set_flip_h(false)
	
	if texture.is_flipped_v():
		texture.set_flip_v(false)

# 画像を反転する
func flip_image(main_node, mirror_node, is_flip_h):
	if is_flip_h:
		if main_node.is_flipped_h():
			mirror_node.set_flip_h(false)
			main_node.set_flip_h(false)
		else:
			mirror_node.set_flip_h(true)
			main_node.set_flip_h(true)
	else:
		if main_node.is_flipped_v():
			mirror_node.set_flip_h(false)
			main_node.set_flip_v(false)
		else:
			mirror_node.set_flip_h(true)
			main_node.set_flip_v(true)

func clear_images():
	bg_image.set_texture(null)
	left_image.set_texture(null)
	right_image.set_texture(null)
	center_image.set_texture(null)
	left_emote.set_texture(null)
	right_emote.set_texture(null)
	center_emote.set_texture(null)

func add_page():
	page = str(int(page) + 1)
	### debug mode
	debug_page_label.set_text(page)

func debug_utility():
	if !debug_page_debugger.visible:
		debug_page_debugger.show()
	else:
		debug_page_debugger.hide()

func debug_back_page():
	if int(page) > 1:
		page = str(int(page) -1)
		debug_page_label.set_text(page)

func debug_next_page():
	add_page()
	debug_page_label.set_text(page)

func debug_page_update():
	page = debug_page_label.get_text()
