extends Node
# テキスト主体の簡易演出用フレームワーク StoryLib
# 他の関連スクリプトにextendsしてお使いください

#########
# 変数群 #
#########

### プロジェクト毎に個別設定をする項目 ####
# ソフトウェアバージョン
const app_version = "1.1.0"
# デバッグモードの切り替え
const is_story_debug_mode = false
# セーブの暗号化を使用する(trueを推奨)
# ※注意: この設定を変更すると、既存のセーブデータが正しく認識されなくなります
const use_encrypt : = true
# 複合化キー
# ※注意: この設定を変更すると、既存のセーブデータが正しく認識されなくなります
const encrypt_key = "encrypt123456789"
# ファイルのデータがセーブデータであることを認識する文字列
# ※注意: この設定を変更すると、既存のセーブデータが正しく認識されなくなります
const for_takeover_namespace = "SAVES"
# スキップボタンを有効にする
const enabled_skip_button = true

# 入出力用セーブデータ
# 任意のデータを用意することができます
# ※注意: この設定を変更すると、既存のセーブデータが正しく認識されなくなります
func saves():
	var save = {
		"location" : "", # 現在地を認識するための変数(変更or削除しないことを推奨)
		"enabled_skip" : true, # スキップボタン確認用
		# 以下テスト用
		"switch_bool" : false,
		"switch_int" : 0,
		"switch_string" : "Variable String!",
	}
	return save
### 個別設定ここまで ####################

# セーブファイルのルートディレクトリ
const root = "user://"
# セーブデータの拡張子
const extension = "json"
# ゲーム内テキストデータの読み込み元
const text_directory = "res://Stories"
# ローカルストレージのセーブファイル名
const save_file_name = "saves"
# タイマー系
const t : = "timeout"
const fadein_timer : = float(2.0)
# 行頭禁則文字(エラーが出た文字は含まず) - 未使用
const kinsoku_words_begin = [",", ")", "]", "｝", "、", "〕", 
"〉", "》", "」", "』", "】", "〟", "’", "”｠", "»", 
"ゝ", "ゞ", "ー", "ァ", "ィ", "ゥ", "ェ", "ォ", "ッ", "ャ", 
"ュ", "ョ", "ヮ", "ヵ", "ヶ", "ぁ", "ぃ", "ぅ", "ぇ", "ぉ",
"っ", "ゃ", "ゅ", "ょ", "‐゠", "–", "〜", "～", "?", "!", 
"‼", "⁇", "⁈", "⁉", "・", ":", ";", "/", "。", ".", "！", "？"]

# saves()を格納する一時セーブデータ
var temporary_saves
# 引き継がれたセーブデータ
var taken_over_saves = {}
# テキストのスムーズ表示を有効にする
var enabled_text_smooth_animation = true
# 現在のストーリ（Dataから代入して使う）
var current_story = ""
var page = "1"
# スキップ用のマーカー変数
var skip_marker : String = ""
# 選択肢の分岐に使用するための変数
# ※このバージョンでは最大同時6つの選択肢を表示することができます
var jump_marker_is_valid : bool = false
var jump_marker : String = ""
var choice_marker1 : String = ""
var choice_marker2 : String = ""
var choice_marker3 : String = ""
var choice_marker4 : String = ""
var choice_marker5 : String = ""
var choice_marker6 : String = ""

#########
# 関数群 #
#########

### データ管理 ###
# セーブデータをロードする
# ※継承のreadyにシーン毎に記述する
func load_save():
	# セーブを一時ファイルにコピーし、一時ファイルから読み込み
	if bool_find_file("save", save_file_name):
		temporary_saves = get_read("save", save_file_name)
		
		if is_story_debug_mode:
			print("debug: ", "successed load save data.")
	# セーブがなければローカルストレージに新規作成
	else:
		write(save_file_name, saves())
		temporary_saves = saves()
		
		if is_story_debug_mode:
			print("debug: ", "successed create new save data.")

# 一時セーブデータの特定の項目を呼び出す
func get_temporary(column):
	return temporary_saves[column]

# 指定したjsonデータの特定の項目を呼び出す
func get_field(file_type, file_name, column):
	var file = File.new()
	var data = get_read(file_type, file_name)
	var field
	
	if bool_find_file(file_type, file_name):
		if typeof(data) == 18:
			for i in data.keys():
				match i:
					column:
						field = data[i]
	
	return field

# 指定のデータを読み込む
func get_read(file_type, file_name):
	var file = File.new()
	var path
	var dict
	
	match file_type:
		"story":
			path = text_directory + "/" + file_name + "." + extension
		"save":
			path = root + "/" + file_name + "." + extension
		"save_copy":
			path = root + "/" + file_name + "." + extension
	
	if file.file_exists(path):
		match file_type:
			# ストーリーを開く
			"story":
				file.open(path, File.READ)
			# セーブデータを開く
			"save":
				if use_encrypt:
					file.open_encrypted_with_pass(path, File.READ, encrypt_key)
				else:
					file.open(path, File.READ)
			# セーブデータをクリップボードにコピーする
			"save_copy":
				if use_encrypt:
					file.open_encrypted_with_pass(path, File.READ, encrypt_key)
				else:
					file.open(path, File.READ)
				
				OS.set_clipboard(for_takeover_namespace + str(Marshalls.variant_to_base64(file.get_line())))
				file.close()
				return
		
		dict = parse_json(file.get_line())
		file.close()
	
	return dict

# ストーリーテキストの指定の行を取得する
func get_line(file_name, num):
	var file = File.new()
	var path_txt = text_directory + "/" + file_name + ".txt"
	var dict
	
	if file.file_exists(path_txt):
		file.open(path_txt, File.READ)
		dict = file.get_as_text().split("\n")[int(num)]
		file.close()
	
	return dict

# 指定のファイルが存在するか確認する
func bool_find_file(file_type, file_name):
	var file = File.new()
	var directory
	
	match file_type:
		"story":
			directory = text_directory
		"save":
			directory = root
	
	if file.file_exists(directory + "/" + str(file_name) + "." + extension):
		return true
	else:
		return false

# ファイルの新規作成または上書き
func write(file_name, update_data):
	var file = File.new()
	var directory = root + file_name + "." + extension
	
	if use_encrypt:
		file.open_encrypted_with_pass(directory, File.WRITE, encrypt_key)
	else:
		file.open(directory, File.WRITE)
	
	file.store_line(to_json(update_data))
	file.close()

# 特定のセーブデータを更新する
func update_save(column, update_data):
	var file = File.new()
	var data = get_read("save", save_file_name)
	
	if bool_find_file("save", save_file_name):
		if typeof(data) == 18:
			for i in data.keys():
				match i:
					column:
						data[i] = update_data
	
	write(save_file_name, data)

# セーブデータを削除する
func clear_save():
	var dir = Directory.new()
	var path = root + "/" + save_file_name + "." + extension
	
	if bool_find_file("save", save_file_name):
		dir.remove(path) # セーブファイル
		# セーブデータの作成に関する不具合回避のためコメントアウト
		# これにより、フォルダは残したままセーブファイルのみ削除される
		# dir.remove(root) # セーブフォルダ
		
		if is_story_debug_mode:
			print("debug: ", "successed clear save data.")

# 引継ぎ可能なセーブデータかどうか確認する - 1.1.0
func check_taken_over_save(base_save, new_save):
	if new_save.left(for_takeover_namespace.length()) != for_takeover_namespace:
		return false
	else:
		new_save = new_save.lstrip(new_save.left(for_takeover_namespace.length()))
		taken_over_saves = parse_json(Marshalls.base64_to_variant(new_save))
		
		if saves().size() == taken_over_saves.size():
			return true
		else:
			return false

### ストーリー関連 ###
# イメージのフェードイン
func image_fadein(main_node, mirror_node, texture, anim_node, anim_name):
	mirror_node.modulate = Color(255, 255, 255, 0)
	mirror_node.set_texture(Data.get_image(texture))
	anim_node.play(anim_name)

# イメージファイルをセットする
func set_image(main_node, mirror_node, texture):
	main_node.set_texture(Data.get_image(texture))
	mirror_node.set_texture(null)

# ランダム整数の取得
func get_random(int_min, int_max): 
	randomize() # ランダム数字を扱う際はこれを必ず記述する
	var random = int(rand_range(int_min, int_max + 1))
	return random

# 指定string_strの末尾から数えてint_num番目の文字を取得
func get_last(string_str, int_num):
	return string_str.lstrip(string_str.left(string_str.length() - int_num))
